@@ -20,6 +20,12 @@ use crate::auth::AuthType;
 use crate::formatter::Formatter;
 #[cfg(feature = "lipsum")]
 use lipsum::lipsum;
+use lazy_static::lazy_static;
+
+lazy_static! {
+    static ref CLRF_MATCH: Regex = Regex::new(r#"\r\n"#).unwrap();
+    static ref CHOMP_MATCH : Regex = Regex::new(r#"\n\s*\n"#).expect("Failed to crate chomp regex");
+}
 
 /// Type signature of basic macros
 ///
@@ -65,11 +71,16 @@ impl BasicMacro {
         #[allow(unused_mut)]
         let mut map = HashMap::from_iter(IntoIter::new([
             ("-".to_owned(),       BasicMacro::get_pipe         as MacroType),
+            ("abs".to_owned(),     BasicMacro::absolute_path    as MacroType),
             ("append".to_owned(),  BasicMacro::append           as MacroType),
+            ("arr".to_owned(),     BasicMacro::array            as MacroType),
+            ("assert".to_owned(),  BasicMacro::assert           as MacroType),
+            ("nassert".to_owned(), BasicMacro::assert_ne        as MacroType),
             ("bind".to_owned(),    BasicMacro::bind_to_local    as MacroType),
             ("chomp".to_owned(),   BasicMacro::chomp            as MacroType),
             ("comp".to_owned(),    BasicMacro::compress         as MacroType),
             ("env".to_owned(),     BasicMacro::get_env          as MacroType),
+            ("envset".to_owned(),  BasicMacro::set_env          as MacroType),
             ("fileout".to_owned(), BasicMacro::file_out         as MacroType),
             ("global".to_owned(),  BasicMacro::global           as MacroType),
             ("include".to_owned(), BasicMacro::include          as MacroType),
@@ -79,14 +90,12 @@ impl BasicMacro {
             ("nl".to_owned(),      BasicMacro::newline          as MacroType),
             ("parent".to_owned(),  BasicMacro::get_parent       as MacroType),
             ("path".to_owned(),    BasicMacro::merge_path       as MacroType),
-            ("paths".to_owned(),   BasicMacro::merge_path_vec   as MacroType),
             ("pipe".to_owned(),    BasicMacro::pipe             as MacroType),
             ("read".to_owned(),    BasicMacro::read             as MacroType),
             ("redir".to_owned(),   BasicMacro::temp_redirect    as MacroType),
             ("regex".to_owned(),   BasicMacro::regex_sub        as MacroType),
             ("rename".to_owned(),  BasicMacro::rename_call      as MacroType),
             ("repeat".to_owned(),  BasicMacro::repeat           as MacroType),
-            ("repl".to_owned(),    BasicMacro::replace          as MacroType),
             ("sub".to_owned(),     BasicMacro::substring        as MacroType),
             ("syscmd".to_owned(),  BasicMacro::syscmd           as MacroType),
             ("tempin".to_owned(),  BasicMacro::temp_include     as MacroType),
@@ -261,10 +270,11 @@ impl BasicMacro {
     fn chomp(args: &str, greedy: bool, processor: &mut Processor) -> Result<Option<String>, RadError> {
         if let Some(args) = ArgParser::new().args_with_len(args, 1, greedy) {
             let source = &args[0];
-            let reg = Regex::new(&format!(r"{0}\s*{0}", &processor.newline))?;
-            let result = reg.replace_all(source, &format!("{0}{0}", &processor.newline));
+            // First convert all '\r\n' into '\n' and reformat it into current newline characters
+            let lf_converted = &*CLRF_MATCH.replace_all(source, r#"\n"#);
+            let chomp_result = &*CHOMP_MATCH.replace_all(lf_converted, format!("{0}{0}",&processor.state.newline));
 
-            Ok(Some(result.to_string()))
+            Ok(Some(chomp_result.to_string()))
         } else {
             Err(RadError::InvalidArgument("Chomp requires an argument".to_owned()))
         }
@@ -327,9 +337,9 @@ impl BasicMacro {
 
             // if current input is not stdin and file path is relative
             // Create new file path that starts from current file path
-            if processor.current_input != "stdin" && file_path.is_relative() {
+            if processor.state.current_input != "stdin" && file_path.is_relative() {
                 // It is ok get parent because any path that has a length can return parent
-                file_path = PathBuf::from(&processor.current_input).parent().unwrap().join(file_path);
+                file_path = PathBuf::from(&processor.state.current_input).parent().unwrap().join(file_path);
             }
 
             if file_path.is_file() { 
@@ -365,9 +375,9 @@ impl BasicMacro {
 
             // if current input is not stdin and file path is relative
             // Create new file path that starts from current file path
-            if processor.current_input != "stdin" && file_path.is_relative() {
+            if processor.state.current_input != "stdin" && file_path.is_relative() {
                 // It is ok get parent because any path that has a length can return parent
-                file_path = PathBuf::from(&processor.current_input).parent().unwrap().join(file_path);
+                file_path = PathBuf::from(&processor.state.current_input).parent().unwrap().join(file_path);
             }
 
             if file_path.is_file() { 
@@ -470,10 +480,78 @@ impl BasicMacro {
     }
 
     /// Placeholder for define
-    fn define_type(_: &str, _dy: bool, _essor: &mut Processor) -> Result<Option<String>, RadError> {
+    fn define_type(_: &str, _: bool, _: &mut Processor) -> Result<Option<String>, RadError> {
         Ok(None)
     }
 
+
+
+    /// Array
+    ///
+    /// # Usage
+    ///
+    /// $arr(1 2 3)
+    fn array(args: &str, _: bool, _: &mut Processor) -> Result<Option<String>, RadError> {
+        let parsed = ArgParser::new().args_to_vec(args, ',', GreedyState::Never);
+        if parsed.len() == 0 {
+            Err(RadError::InvalidArgument("Array requires an argument".to_owned()))
+        } else {
+            let separater = if parsed.len() >= 2 {
+                &parsed[1] // Use given separater
+            } else { " " }; // Use whitespace as default
+            let mut vec = parsed[0].split(separater).collect::<Vec<&str>>();
+
+            // Also given filter argument, then filter with regex expression
+            if parsed.len() == 3 {
+                let reg = Regex::new(&parsed[2])?;
+                vec = vec.into_iter().filter(|&item| reg.is_match(item)).collect();
+            }
+
+            // Join as csv
+            let joined = vec.join(",");
+
+            Ok(Some(joined))
+        }
+    }
+
+    /// Assert
+    ///
+    /// # Usage
+    ///
+    /// $assert(abc,abc)
+    fn assert(args: &str, greedy: bool, p: &mut Processor) -> Result<Option<String>, RadError> {
+        if let Some(args) = ArgParser::new().args_with_len(args, 2, greedy) {
+            if args[0] == args[1] {
+                p.track_assertion(true)?;
+                Ok(None)
+            } else {
+                p.track_assertion(false)?;
+                Err(RadError::AssertFail)
+            }
+        } else {
+            Err(RadError::InvalidArgument("Assert requires two arguments".to_owned()))
+        }
+    }
+
+    /// Assert not equal
+    ///
+    /// # Usage
+    ///
+    /// $nassert(abc,abc)
+    fn assert_ne(args: &str, greedy: bool, p: &mut Processor) -> Result<Option<String>, RadError> {
+        if let Some(args) = ArgParser::new().args_with_len(args, 2, greedy) {
+            if args[0] != args[1] {
+                p.track_assertion(true)?;
+                Ok(None)
+            } else {
+                p.track_assertion(false)?;
+                Err(RadError::AssertFail)
+            }
+        } else {
+            Err(RadError::InvalidArgument("Assert_ne requires two arguments".to_owned()))
+        }
+    }
+
     /// Create multiple macro executions from given csv value
     ///
     /// # Usage
@@ -493,9 +571,32 @@ impl BasicMacro {
             // Plus, it is already trimmed by csv crate.
             let macro_data = &args[1];
 
-            let result = Formatter::csv_to_macros(&macro_name, macro_data, &processor.newline)?;
-            // This is necessary
+            let result = Formatter::csv_to_macros(&macro_name, macro_data, &processor.state.newline)?;
+
+            // TODO 
+            // This behaviour might can be improved
+            // Disable debugging for nested macro expansion
+            #[cfg(feature = "debug")]
+            let original = processor.is_debug();
+            processor.debug(false);
+
+            // Parse macros
             let result = processor.parse_chunk_args(0, "", &result)?;
+
+            // Set custom prompt log to indicate user thatn from macro doesn't support
+            // debugging inside macro expansion
+            #[cfg(feature = "debug")]
+            {
+                use crate::debugger::DebugSwitch;
+                processor.debug(original);
+                match processor.get_debug_switch() {
+                    DebugSwitch::StepMacro | DebugSwitch::NextMacro => {
+                        processor.set_prompt_log("\"From macro\"")
+                    }
+                    _ => ()
+                }
+            }
+
             Ok(Some(result))
         } else {
             Err(RadError::InvalidArgument("From requires two arguments".to_owned()))
@@ -515,7 +616,7 @@ impl BasicMacro {
         if let Some(args) = ArgParser::new().args_with_len(args, 2, greedy) {
             let table_format = &args[0]; // Either gfm, wikitex, latex, none
             let csv_content = &args[1];
-            let result = Formatter::csv_to_table(table_format, csv_content, &p.newline)?;
+            let result = Formatter::csv_to_table(table_format, csv_content, &p.state.newline)?;
             Ok(Some(result))
         } else {
             Err(RadError::InvalidArgument("Table requires two arguments".to_owned()))
@@ -531,7 +632,7 @@ impl BasicMacro {
     /// $pipe(Value)
     fn pipe(args: &str, greedy: bool, processor: &mut Processor) -> Result<Option<String>, RadError> {
         if let Some(args) = ArgParser::new().args_with_len(args, 1, greedy) {
-            processor.pipe_value = args[0].to_owned();
+            processor.state.pipe_value = args[0].to_owned();
         }
         Ok(None)
     }
@@ -549,8 +650,10 @@ impl BasicMacro {
                 processor.log_warning(&format!("Creating a global with a name already existing : \"{}\"", name))?;
             }
             processor.add_custom_rules(vec![(name,"",value)]);
+            Ok(None)
+        } else {
+            Err(RadError::InvalidArgument("Global requires two argument".to_owned()))
         }
-        Ok(None)
     }
 
     /// Bind a local macro
@@ -568,8 +671,10 @@ impl BasicMacro {
             if processor.get_map().contains(name) {
                 processor.log_warning(&format!("Creating a binding with a name already existing : \"{}\"", name))?;
             }
+            Ok(None)
+        } else {
+            Err(RadError::InvalidArgument("Bind requires two argument".to_owned()))
         }
-        Ok(None)
     }
 
     /// Get environment variable with given name
@@ -584,29 +689,34 @@ impl BasicMacro {
         if let Ok(out) = std::env::var(args) {
             Ok(Some(out))
         } else { 
-            if p.strict {
+            if p.state.strict {
                 p.log_warning(&format!("Env : \"{}\" is not defined.", args))?;
             }
             Ok(None) 
         }
     }
 
-    /// Merge two path into a single path
-    ///
-    /// This creates platform agonistic path which can be consumed by other macros.
+    /// Set environment variable with given name
     ///
     /// # Usage
     ///
-    /// $path($env(HOME),document)
-    fn merge_path(args: &str, greedy: bool, _: &mut Processor) -> Result<Option<String>, RadError> {
+    /// $envset(SHELL,value)
+    fn set_env(args: &str, greedy: bool, p : &mut Processor) -> Result<Option<String>, RadError> {
+        if !Utils::is_granted("envset", AuthType::ENV,p)? {
+            return Ok(None);
+        }
         if let Some(args) = ArgParser::new().args_with_len(args, 2, greedy) {
-            let target = Utils::trim(&args[0]);
-            let added = Utils::trim(&args[1]);
+            let name = &args[0];
+            let value = &args[1];
 
-            let out = format!("{}",&std::path::Path::new(&target).join(&added).display());
-            Ok(Some(out))
+            if p.state.strict && std::env::var(name).is_ok() {
+                return Err(RadError::InvalidArgument(format!("You cannot override environment variable in strict mode. Failed to set \"{}\"", name)));
+            }
+
+            std::env::set_var(name, value);
+            Ok(None)
         } else {
-            Err(RadError::InvalidArgument("Path macro needs two arguments".to_owned()))
+            Err(RadError::InvalidArgument("Envset requires two arguments".to_owned()))
         }
     }
 
@@ -616,8 +726,8 @@ impl BasicMacro {
     ///
     /// # Usage
     ///
-    /// $paths($env(HOME) document test.docx)
-    fn merge_path_vec(args: &str, _: bool, _: &mut Processor) -> Result<Option<String>, RadError> {
+    /// $path($env(HOME),document,test.docx)
+    fn merge_path(args: &str, _: bool, _: &mut Processor) -> Result<Option<String>, RadError> {
         let vec = ArgParser::new().args_to_vec(args, ',', GreedyState::Never);
 
         let out = vec
@@ -638,7 +748,7 @@ impl BasicMacro {
     ///
     /// $nl()
     fn newline(_: &str, _: bool, p: &mut Processor) -> Result<Option<String>, RadError> {
-        Ok(Some(p.newline.to_owned()))
+        Ok(Some(p.state.newline.to_owned()))
     }
 
     /// Get name from given path
@@ -662,6 +772,25 @@ impl BasicMacro {
         }
     }
 
+    /// Get absolute path from given path
+    ///
+    /// # Usage
+    ///
+    /// $abs(../canonic_path.txt)
+    fn absolute_path(args: &str, _: bool, p: &mut Processor) -> Result<Option<String>, RadError> {
+        if !Utils::is_granted("abs", AuthType::FIN, p)? {
+            return Ok(None);
+        }
+
+        if let Some(args) = ArgParser::new().args_with_len(args, 1, false) {
+            let path = Path::new(&args[0]);
+            let canonic = std::fs::canonicalize(path)?.to_str().unwrap().to_owned();
+            Ok(Some(canonic))
+        } else {
+            Err(RadError::InvalidArgument("Abs requires an argument".to_owned()))
+        }
+    }
+
     /// Get parent from given path
     ///
     /// # Usage
@@ -689,7 +818,7 @@ impl BasicMacro {
     ///
     /// $-()
     fn get_pipe(_: &str, _: bool, processor: &mut Processor) -> Result<Option<String>, RadError> {
-        let out = processor.pipe_value.clone();
+        let out = processor.state.pipe_value.clone();
         Ok(Some(out))
     }
 
@@ -917,7 +1046,7 @@ impl BasicMacro {
             } else {
                 return Err(RadError::InvalidArgument(format!("Redir's agument should be valid boolean value but given \"{}\"", &args[0])));
             };
-            p.redirect = toggle;
+            p.state.redirect = toggle;
             Ok(None)
         } else {
             Err(RadError::InvalidArgument("Redir requires an argument".to_owned()))
@@ -940,22 +1069,4 @@ impl BasicMacro {
             Err(RadError::InvalidArgument("Temp requires an argument".to_owned()))
         }
     }
-
-    /// Replace value
-    ///
-    /// # Usage
-    ///
-    /// $repl(macro,value)
-    fn replace(args: &str, greedy: bool, processor: &mut Processor) -> Result<Option<String>, RadError> {
-        if let Some(args) = ArgParser::new().args_with_len(args, 2, greedy) {
-            let name = args[0].as_str();
-            let target = args[1].as_str();
-            if !processor.get_map().replace(name, target) {
-                return Err(RadError::InvalidArgument(format!("{} doesn't exist, thus cannot replace it's content", name)))
-            }
-            Ok(None)
-        } else {
-            Err(RadError::InvalidArgument("Replace requires two arguments".to_owned()))
-        }
-    }
 }
