@@ -46,11 +46,13 @@ impl Cli {
         self.parse_options(args);
         // Build processor
         let mut processor = Processor::new()
+            .comment(args.is_present("comment"))
             .purge(args.is_present("purge"))
             .greedy(args.is_present("greedy"))
             .lenient(args.is_present("lenient"))
             .silent(args.is_present("silent"))
             .nopanic(args.is_present("nopanic"))
+            .assert(args.is_present("assert"))
             .allow(std::mem::replace(&mut self.allow_auth,None))
             .allow_with_warning(std::mem::replace(&mut self.allow_auth_warn,None))
             .unix_new_line(args.is_present("newline"))
@@ -161,7 +163,7 @@ impl Cli {
     /// Creates argument template wich clap macro
     fn args_builder() -> clap::ArgMatches {
         clap_app!(rad =>
-            (version: "0.11.3")
+            (version: "1.1.0")
             (author: "Simon Creek <simoncreek@tutanota.com>")
             (about: "R4d is a modern macro processor made with rust")
             (@arg FILE: ... "Files to execute processing")
@@ -173,17 +175,19 @@ impl Cli {
             (@arg purge: -p --purge "Purge unused macros")
             (@arg lenient: -l --lenient "Lenient mode, disables strict mode")
             (@arg nopanic: --nopanic "Don't panic in any circumstances, the most lenient mode")
+            (@arg assert: --assert "Enable assert mode")
             (@arg debug: -d --debug "Debug mode")
             (@arg log: --log "Debug log mode")
             (@arg diff: --diff "Show diff result")
+            (@arg comment: --comment "Use comment option")
             (@arg interactive: -i --interactive "Use interactive debug mode")
             (@arg combination: -c "Read from both stdin and file inputs")
-            (@arg discard: -D --discard "Discard output without print out")
+            (@arg discard: -D --discard "Discard output")
             (@arg allow: -a +takes_value "Allow permission (fin|fout|cmd|env)")
             (@arg allow_warn: -w +takes_value "Allow permission with warnings (fin|fout|cmd|env)")
             (@arg allow_all: -A conflicts_with[allow_all_warn] "Allow all permission")
             (@arg allow_all_warn: -W "Allow all permission with warning")
-            (@arg silent: -s "Supress warnings")
+            (@arg silent: -s --silent "Supress warnings")
             (@arg newline: -n "Use unix newline for formatting")
         ).get_matches()
     }
