@@ -2,11 +2,7 @@
 //!
 //! Logger handles all kinds of logging logics. Such log can be warning, error or debug logs.
 
-#[cfg(feature = "debug")]
-use std::io::BufRead;
 use std::io::Write;
-#[cfg(feature = "debug")]
-use crossterm::{ExecutableCommand, terminal::ClearType};
 use crate::models::WriteOption;
 use crate::consts::*;
 use crate::utils::Utils;
@@ -23,9 +19,10 @@ pub(crate) struct Logger {
     suppress_warning: bool,
     error_count: usize,
     warning_count: usize,
+    assert_success: usize,
+    assert_fail: usize,
     chunked: usize,
-    #[cfg(feature = "debug")]
-    debug_interactive: bool,
+    pub(crate) assert: bool,
 }
 /// Struct specifically exists to backup information of logger
 #[derive(Debug)]
@@ -48,20 +45,25 @@ impl Logger{
             suppress_warning: false,
             error_count:0,
             warning_count:0,
+            assert_success : 0,
+            assert_fail: 0, 
             chunked : 0,
-            #[cfg(feature = "debug")]
-            debug_interactive: false,
+            assert: false,
         }
     }
 
+    pub fn assert(&mut self) {
+        self.assert = true;
+    }
+
     pub fn suppress_warning(&mut self) {
         self.suppress_warning = true;
     }
 
     /// Enables "chunk" mode whtin logger
     ///
-    /// If chunk mode is enabled line_number doesn't mean real line number, rather it means how
-    /// much lines has passed since last_line_number.
+    /// If chunk mode is enabled line_number doesn't mean real line number, 
+    /// rather it means how much lines has passed since last_line_number.
     pub fn set_chunk(&mut self, switch: bool) {
         if switch {
             self.chunked = self.chunked + 1;
@@ -72,11 +74,6 @@ impl Logger{
         }
     }
 
-    #[cfg(feature = "debug")]
-    pub fn set_debug_interactive(&mut self) {
-        self.debug_interactive = true;
-    }
-
     pub fn set_write_options(&mut self, write_option: Option<WriteOption>) {
         self.write_option = write_option; 
     }
@@ -152,6 +149,8 @@ impl Logger{
     /// Log error
     pub fn elog(&mut self, log : &str) -> Result<(), RadError> {
         self.error_count = self.error_count + 1; 
+
+        if self.assert { return Ok(()); }
         let last_char = self.try_get_last_char();
         if let Some(option) = &mut self.write_option {
             match option {
@@ -185,10 +184,28 @@ impl Logger{
         Ok(())
     }
 
+    #[cfg(feature = "debug")]
+    pub fn elog_no_prompt(&mut self, log : impl std::fmt::Display) -> Result<(), RadError> {
+        if let Some(option) = &mut self.write_option {
+            match option {
+                WriteOption::File(file) => {
+                    file.write_all(log.to_string().as_bytes())?;
+                }
+                WriteOption::Terminal => {
+                    eprint!("{}",log);
+                }
+                WriteOption::Discard => ()
+            } // match end
+        } 
+        Ok(())
+    }
+
     /// Log warning
     pub fn wlog(&mut self, log : &str) -> Result<(), RadError> {
         if self.suppress_warning { return Ok(()); }
         self.warning_count = self.warning_count + 1; 
+
+        if self.assert { return Ok(()); }
         let last_char = self.try_get_last_char();
         if let Some(option) = &mut self.write_option {
             match option {
@@ -222,6 +239,44 @@ impl Logger{
         Ok(())
     }
 
+    /// Assertion log
+    pub fn alog(&mut self, success: bool) -> Result<(), RadError> {
+        if success { 
+            self.assert_success = self.assert_success + 1; 
+            return Ok(());
+        } 
+        self.assert_fail = self.assert_fail + 1; 
+        let last_char = self.try_get_last_char();
+
+        if let Some(option) = &mut self.write_option {
+            match option {
+                WriteOption::File(file) => {
+                    file.write_all(
+                        format!(
+                            "assert fail -> {}:{}:{}{}",
+                            self.current_file,
+                            self.last_line_number,
+                            last_char,
+                            LINE_ENDING
+                        ).as_bytes()
+                    )?;
+                }
+                WriteOption::Terminal => {
+                    eprintln!(
+                        "{} -> {}:{}:{}", 
+                        Utils::red("assert fail"),
+                        self.current_file,
+                        self.last_line_number,
+                        last_char
+                    );
+                }
+                WriteOption::Discard => ()
+            } // match end
+        } 
+
+        Ok(())
+    }
+
     /// Print result of logging of warnings and errors
     pub fn print_result(&mut self) -> Result<(), RadError> {
         if let Some(option) = &mut self.write_option {
@@ -229,18 +284,24 @@ impl Logger{
             if self.error_count == 0 && self.warning_count == 0 {
                 return Ok(())
             }
-    
             // There is either error or warning
             let error_result = format!("{}: found {} errors",Utils::red("error"), self.error_count);
             let warning_result = format!("{}: found {} warnings",Utils::yellow("warning"), self.warning_count);
+            let assert_result = format!("
+{}
+SUCCESS : {}
+FAIL: {}",Utils::green("Assert"), self.assert_success, self.assert_fail
+            );
             match option {
                 WriteOption::File(file) => {
                     if self.error_count > 0 {file.write_all(error_result.as_bytes())?;}
                     if self.warning_count > 0 {file.write_all(warning_result.as_bytes())?;}
+                    if self.assert{ file.write_all(assert_result.as_bytes())?; }
                 }
                 WriteOption::Terminal => {
                     if self.error_count > 0 { eprintln!("{}",error_result);}
                     if self.warning_count > 0 {eprintln!("{}",warning_result);}
+                    if self.assert {eprintln!("{}",assert_result);}
                 }
                 WriteOption::Discard => ()
             }
@@ -272,85 +333,23 @@ impl Logger{
         }
     }
 
-    /// Log debug information
-    #[cfg(feature = "debug")]
-    pub fn dlog_command(&self, log : &str, prompt: Option<&str>) -> Result<String, RadError> {
-        // Disable line wrap
-        if self.debug_interactive {
-            std::io::stdout()
-                .execute(crossterm::terminal::DisableLineWrap)?;
-        }
-
-        let mut input = String::new();
-        let prompt = if let Some(content) = prompt { content } else { "" };
-        eprintln!("{} : {}",Utils::green(&format!("({})", &prompt)), log);
-        print!(">> ");
-
-        // Restore wrapping
-        if self.debug_interactive {
-            std::io::stdout()
-                .execute(crossterm::terminal::EnableLineWrap)?;
-        }
-        // Flush because print! is not "printed" yet
-        std::io::stdout().flush()?;
-
-        // Get user input
-        let stdin = std::io::stdin();
-        stdin.lock().read_line(&mut input)?;
-        if self.debug_interactive {
-            // Clear user input line
-            // Preceding 1 is for "(input)" prompt
-            self.remove_terminal_lines(1 + Utils::count_sentences(log))?;
-        }
-
-        Ok(input)
-    }
-
     /// Log debug information
     #[cfg(feature = "debug")]
     pub fn dlog_print(&mut self, log : &str) -> Result<(), RadError> {
         if let Some(option) = &mut self.write_option {
             match option {
                 WriteOption::Terminal => {
-                    eprint!("{}{}{}", Utils::green(&format!("{}:{}", self.last_line_number,"log")),LINE_ENDING,log);
+                    eprint!("{}{}{}", Utils::green(&format!("{}:log", self.last_line_number)),LINE_ENDING,log);
                 }
                 WriteOption::File(file) => {
-                    file.write_all(format!("{}:log :{}{}", self.last_line_number,LINE_ENDING,log).as_bytes())?;
+                    file.write_all(format!("{}:log{}{}", self.last_line_number,LINE_ENDING,log).as_bytes())?;
                 }
                 _ => ()
             }
         }
         Ok(())
     }
-
-    #[cfg(feature = "debug")]
-    fn remove_terminal_lines(&self, count: usize) -> Result<(), RadError> {
-
-        std::io::stdout()
-            .execute(crossterm::terminal::Clear(ClearType::CurrentLine))?;
-
-        // Range is max exclusive thus min should start from 0
-        // e.g. 0..1 only tries once with index 0
-        for _ in 0..count {
-            std::io::stdout()
-                .execute(crossterm::cursor::MoveUp(1))?
-                .execute(crossterm::terminal::Clear(ClearType::CurrentLine))?;
-        }
-
-        Ok(())
-    }
-
     // End of debug related methods
     // </DEBUG>
     // ----------
 } 
-
-/// Debug switch(state) that indicates what debugging behaviours are intended for next branch
-#[cfg(feature = "debug")]
-pub enum DebugSwitch {
-    UntilMacro,
-    NextLine,
-    NextMacro,
-    StepMacro,
-    NextBreakPoint(String),
-}
